@@ -8,7 +8,7 @@ from joblib import delayed, Parallel
 
 
 def enrichment_score(gene_list, correl_vector, gene_set, weighted_score_type=1, 
-                     nperm=1000, rs=None, single=False, scale=False):
+                     nperm=1000, seed=None, single=False, scale=False):
     """This is the most important function of GSEApy. It has the same algorithm with GSEA and ssGSEA.
 
     :param gene_list:       The ordered gene list gene_name_list, rank_metric.index.values
@@ -26,7 +26,7 @@ def enrichment_score(gene_list, correl_vector, gene_set, weighted_score_type=1,
                             the gene list. Or rankings, rank_metric.values
     :param nperm:           Only use this parameter when computing esnull for statistical testing. Set the esnull value
                             equal to the permutation number.
-    :param rs:              Random state for initializing gene list shuffling. Default: seed=None
+    :param seed:            Random state for initializing gene list shuffling. Default: seed=None
 
     :return:
 
@@ -59,7 +59,7 @@ def enrichment_score(gene_list, correl_vector, gene_set, weighted_score_type=1,
     tag_indicator = np.tile(tag_indicator, (nperm+1,1))
     correl_vector = np.tile(correl_vector,(nperm+1,1))
     # gene list permutation
-    rs = np.random.RandomState(rs)
+    rs = np.random.RandomState(seed)
     for i in range(nperm): rs.shuffle(tag_indicator[i])
     # np.apply_along_axis(rs.shuffle, 1, tag_indicator)
 
@@ -87,7 +87,7 @@ def enrichment_score(gene_list, correl_vector, gene_set, weighted_score_type=1,
 
 
 def enrichment_score_tensor(gene_mat, cor_mat, gene_sets, weighted_score_type, nperm=1000,
-                            rs=None, single=False, scale=False):
+                            seed=None, single=False, scale=False):
     """Next generation algorithm of GSEA and ssGSEA. Works for 3d array
 
         :param gene_mat:        the ordered gene list(vector) with or without gene indices matrix.
@@ -99,7 +99,7 @@ def enrichment_score_tensor(gene_mat, cor_mat, gene_sets, weighted_score_type, n
         :param int nperm:       permutation times.
         :param bool scale:      If True, normalize the scores by number of genes_mat.
         :param bool single:     If True, use ssGSEA algorithm, otherwise use GSEA.
-        :param rs:              Random state for initialize gene list shuffling.
+        :param seed:              Random state for initialize gene list shuffling.
                                 Default: seed=None
         :return: a tuple contains::
 
@@ -109,7 +109,7 @@ def enrichment_score_tensor(gene_mat, cor_mat, gene_sets, weighted_score_type, n
                  | RES: The running enrichment score for all locations in the gene list.
 
     """
-    rs = np.random.RandomState(rs)
+    rs = np.random.RandomState(seed)
     # gene_mat -> 1d: prerank, ssSSEA or 2d: GSEA
     keys = sorted(gene_sets.keys())
 
@@ -232,6 +232,8 @@ def ranking_metric_tensor(exprs, method, permutation_num, pos, neg, classes,
     classes = np.array(classes)
     pos = classes == pos
     neg = classes == neg
+    n_pos = np.sum(pos)
+    n_neg = np.sum(neg)
     pos_cor_mean = perm_cor_tensor[:,pos,:].mean(axis=1)
     neg_cor_mean = perm_cor_tensor[:,neg,:].mean(axis=1)
     pos_cor_std = perm_cor_tensor[:,pos,:].std(axis=1, ddof=1)
@@ -242,8 +244,8 @@ def ranking_metric_tensor(exprs, method, permutation_num, pos, neg, classes,
     elif method in ['abs_signal_to_noise', 'abs_s2n']:
         cor_mat = np.abs((pos_cor_mean - neg_cor_mean)/(pos_cor_std + neg_cor_std))
     elif method == 't_test':
-        denom = 1.0/G
-        cor_mat = (pos_cor_mean - neg_cor_mean)/ np.sqrt(denom*pos_cor_std**2 + denom*neg_cor_std**2)
+        denom = np.sqrt((pos_cor_std**2)/n_pos  + (neg_cor_std**2)/n_neg)
+        cor_mat = (pos_cor_mean - neg_cor_mean)/ denom
     elif method == 'ratio_of_classes':
         cor_mat = pos_cor_mean / neg_cor_mean
     elif method == 'diff_of_classes':
@@ -312,13 +314,14 @@ def ranking_metric(df, method, pos, neg, classes, ascending):
     # exclude any zero stds.
     df_mean = df.groupby(by=classes, axis=1).mean()
     df_std =  df.groupby(by=classes, axis=1).std()
-
+    n_pos = np.sum(classes == pos)
+    n_neg = np.sum(classes == neg)
     if method in ['signal_to_noise', 's2n']:
         ser = (df_mean[pos] - df_mean[neg])/(df_std[pos] + df_std[neg])
     elif method in ['abs_signal_to_noise', 'abs_s2n']:
         ser = ((df_mean[pos] - df_mean[neg])/(df_std[pos] + df_std[neg])).abs()
     elif method == 't_test':
-        ser = (df_mean[pos] - df_mean[neg])/ np.sqrt(df_std[pos]**2/len(df_std)+df_std[neg]**2/len(df_std) )
+        ser = (df_mean[pos] - df_mean[neg])/ np.sqrt(df_std[pos]**2/n_pos+df_std[neg]**2/n_neg)
     elif method == 'ratio_of_classes':
         ser = df_mean[pos] / df_mean[neg]
     elif method == 'diff_of_classes':
@@ -466,19 +469,17 @@ def gsea_compute(data, gmt, n, weighted_score_type, permutation_type,
     RES=[]
     hit_ind=[]
     esnull = [ [] for a in range(len(subsets)) ]
-
+    np.random.seed(seed) # control the ranodm numbers
     logging.debug("Start to compute enrichment scores......................")
 
     if permutation_type == "phenotype":
         logging.debug("Start to permutate classes..............................")
-        # shuffling classes and generate random correlation rankings
-        rs = np.random.RandomState(seed)
-        # this version don't split large array into smaller ones
+        # this version won't split large array into smaller ones
         genes_mat, cor_mat = ranking_metric_tensor(exprs=data, method=method,
                                                    permutation_num=n+1,
                                                    pos=pheno_pos, neg=pheno_neg,
                                                    classes=classes,
-                                                   ascending=ascending, rs=rs, 
+                                                   ascending=ascending, seed=seed, 
                                                    skip_last=True)
 
         # compute es, esnulls. hits, RES
@@ -487,7 +488,7 @@ def gsea_compute(data, gmt, n, weighted_score_type, permutation_type,
                                                            cor_mat=cor_mat,
                                                            gene_sets=gmt,
                                                            weighted_score_type=w,
-                                                           nperm=n, rs=rs,
+                                                           nperm=n, seed=seed,
                                                            single=False, scale=False,)
 
     else:
@@ -496,21 +497,19 @@ def gsea_compute(data, gmt, n, weighted_score_type, permutation_type,
         logging.debug("Start to compute es and esnulls........................")
         ## this version don't split large array into smaller ones
         # es, esnull, hit_ind, RES = enrichment_score_tensor(gene_mat=gl,
-        #                                                    cor_mat=cor_mat,
+        #                                                    cor_mat=cor_vec,
         #                                                    gene_sets=gmt,
         #                                                    weighted_score_type=w,
         #                                                    nperm=n, rs=rs
         #                                                    single=single, scale=scale)
-
         temp_esnu=[]
         # you have to reseed, or all your processes are sharing the same seed value
-        np.random.seed(seed)
-        random_state = np.random.randint(np.iinfo(np.int32).max, size=len(subsets))
-
+        # np.random.seed(seed)
+        random_seeds= np.random.randint(np.iinfo(np.int32).max, size=len(subsets))
         temp_esnu = Parallel(n_jobs=processes, require='sharedmem')(delayed(enrichment_score)(
                         gl, cor_vec, gmt.get(subset), w, n, 
                         rs, single, scale) 
-                        for subset, rs in zip(subsets, random_state))        
+                        for subset, rs in zip(subsets, random_seeds))        
         # esn is a list, don't need to use append method.
         for si, temp in enumerate(temp_esnu):
             e, enu, hit, rune = temp
@@ -555,7 +554,6 @@ def normalize(es, esnull):
     esnull_neg = np.ma.MaskedArray(esnull, mask=(esnull>=0)).mean(axis=1)
     esnull_pos = np.array(esnull_pos)
     esnull_neg = np.array(esnull_neg)
-
     # NES
     nEnrichmentScores  = np.where(es>=0, es/esnull_pos, -es/esnull_neg)
     # NES_NULL
